#!/bin/bash
# +------------------------------------------------------------------+
# |          _           _           _       _   __   _______        |
# |       __| |_  ___ __| |__  _ __ | |__   / | /  \ |__ / __|       |
# |      / _| ' \/ -_) _| / / | '  \| / /   | || () | |_ \__ \       |
# |      \__|_||_\___\__|_\_\_|_|_|_|_\_\   |_(_)__(_)___/___/       |
# |                                            check_mk 1.0.35       |
# |                                                                  |
# | Copyright Mathias Kettner 2009                mk@mathias-kettner |
# +------------------------------------------------------------------+
# 
# This file is part of check_mk 1.0.35.
# The official homepage is at http://mathias-kettner.de/check_mk.
# 
# check_mk is free software;  you can redistribute it and/or modify it
# under the  terms of the  GNU General Public License  as published by
# the Free Software Foundation in version 2.  check_mk is  distributed
# in the hope that it will be useful, but WITHOUT ANY WARRANTY;  with-
# out even the implied warranty of  MERCHANTABILITY  or  FITNESS FOR A
# PARTICULAR PURPOSE. See the  GNU General Public License for more de-
# ails.  You should have  received  a copy of the  GNU  General Public
# License along with GNU Make; see the file  COPYING.  If  not,  write
# to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
# Boston, MA 02110-1301 USA.


VERSION=1.0.35
NAME=check_mk
LANG=
LC_ALL=
SETUPCONF=~/.check_mk_setup.conf

# Find path to ourselves
SRCDIR=${0%/*}
if [ "$SRCDIR" = "$0" ] ; then SRCDIR=. ; fi
if [ ! -e "$SRCDIR/setup.sh" ] ; then
    echo "Cannot find location of setup.sh.">&2
    echo "Please call setup.sh with its complete path." >&2
    exit 1
fi

# Install check_mk into user defined locations
# This script is run during packaging for RPM
# and DEB. You can also run it manually for a
# customized setup

if [ $UID = 0 -o -n "$DESTDIR" ] ; then
    ROOT=yes
    if [ "$DESTDIR" = / ] ; then
	DESTDIR=
    fi
else
    ROOT=
    DESTDIR=
fi

SUMMARY=
DIRINFO="# Written by setup of check_mk $VERSION at $(date)"

ask_dir () 
{
    if [ $1 != -d ] ; then
	prefix="$(pwd)/"
    else
	prefix=""
	shift
    fi
    VARNAME=$1
    DEF_ROOT=$2
    DEF_USER=$3
    SHORT=$4
    DESCR=$5

    # maybe variable already set (via environment)
    eval "DIR="'$'"$VARNAME"

    if [ -z "$PATHS_READ" -o -z "$DIR" -o "$FORCE_VERIFY" ]
    then
	if [ "$FORCE_VERIFY" -a -n "$DIR" ] ; then
	    DEF=$DIR
	elif [ "$ROOT" ] ; then
	    DEF=$DEF_ROOT
        else
	    DEF=$DEF_USER
        fi

        if [ -z "$DIR" -o "$FORCE_VERIFY" ] ; then
            # DESTDIR is set? -> always use default
	    if [ -z "$DIR" -a -n "$DESTDIR" ] ; then
		DIR=$DEF
            elif [ -z "$DESTDIR" ] ; then
                echo
		if [ "$FORCE_VERIFY" -a -n "$DIR" ] ; then
		    PRINTOUT="[1;42;37m autodetected [0m --> [1;44;36m"
		else
		    PRINTOUT="[1;44;37m"
		fi
		read -p "[4m$SHORT[0m
$DESCR:
($PRINTOUT$DEF[0m): " DIR
		if [ -z "$DIR" ] ; then
		    DIR=$DEF
		fi
            fi
        fi

	# Handle relative paths
        if [ "${DIR:0:1}" != / ] ; then
	    DIR="$prefix$DIR"
        fi
        eval "$VARNAME='$DIR'"
    fi

    SUMMARY="$SUMMARY
$(printf "[44;37m %-30s [1;45;37m %-39s [0m" "$SHORT" $DIR)"

    DIRINFO="$DIRINFO
$VARNAME='$DIR'"
}


ask_title ()
{
    f="[45;37;1m   %-67s  [0m\n"
    echo 
    echo
    printf "$f" ""
    printf "$f" "$*"
    printf "$f" ""
}

cat <<EOF


[1;44;37m                  _               _                  _                  [0m
[1;44;37m              ___| |__   ___  ___| | __    _ __ ___ | | __              [0m
[1;44;37m             / __| '_ \ / _ \/ __| |/ /   | '_ \` _ \| |/ /              [0m
[1;44;37m            | (__| | | |  __/ (__|   <    | | | | | |   <               [0m
[1;44;37m             \___|_| |_|\___|\___|_|\_\___|_| |_| |_|_|\_\              [0m
[1;44;37m                                     |_____|                            [0m
[1;44;37m                                                                        [0m
[1;45;37m   check_mk setup                  $(printf "%32s" Version:' '$VERSION)     [0m


Welcome to check_mk. This setup will install check_mk into user defined
directories. If you run this script as root, installation paths below
/usr will be suggested. If you run this script as non-root user paths
in your home directory will be suggested. You may override the default
values or just hit enter to accept them. 

Your answers will be saved to $SETUPCONF and will be 
reused when you run the setup of this or a later version again. Please
delete that file if you want to delete your previous answers.

EOF

if [ -z "$DESTDIR" ]
then
    echo "Trying to autodetect details of your Nagios installation..."
    if OUTPUT=$(python $SRCDIR/autodetect.py)
    then
        eval "$OUTPUT"
        echo -n "[1;42;37mFound your running Nagios process, "
        echo "autodetected $(echo "$OUTPUT" | wc -l) settings.[0m"
    fi
fi

if [ -z "$DESTDIR" -a -e "$SETUPCONF" ] && . $SETUPCONF
then
    echo "[1;42;36mYou previous settings have been read from $SETUPCONF.[0m"
    PATHS_READ=true
else
    FORCE_VERIFY=yes
fi


HOMEBASEDIR=$HOME/$NAME

ask_title "Installation directories of check_mk"

ask_dir bindir /usr/bin $HOMEBASEDIR/bin "executables" \
  "Directory where to install executable programs such
as check_mk itself"

ask_dir confdir /etc/$NAME $HOMEBASEDIR "check_mk configuration" \
  "Directory where check_mk looks for its configuration files. An example
configuration file will be installed there"

ask_dir checksdir /usr/share/$NAME/checks $HOMEBASEDIR/checks "check_mk checks" \
  "check_mk's different checks are implemented as small Python
scriptlets that parse and interpret the various output sections
of the agents. Where shall those be installed"

ask_dir modulesdir /usr/share/$NAME/modules $HOMEBASEDIR/modules "check_mk modules" \
  "Directory for main componentents of check_mk itself" 

ask_dir mibsdir /usr/share/snmp/mibs $HOMEBASEDIR/mibs "SNMP mibs" \
  "Directory for SNMP MIB files (for copyright reasons we currently do
not ship any MIB files, though...)" 

ask_dir docdir /usr/share/doc/$NAME $HOMEBASEDIR/doc "documentation" \
  "Documentation about check_mk will be installed here"

ask_dir checkmandir /usr/share/doc/$NAME/checks $HOMEBASEDIR/doc/checks "check manuals" \
  "Directory for manuals for the various checks. The manuals can be viewed 
with check_mk -M <CHECKNAME>".

ask_dir vardir /var/lib/$NAME $HOMEBASEDIR/var "working directory of check_mk" \
  "check_mk will create caches files, automatically created checks and
other files into this directory. The directory needs to be writable by 
the Nagios process" 

ask_dir agentsdir /usr/share/$NAME/agents $HOMEBASEDIR/agents "agents for operating systems" \
  "Agents for various operating systems will be installed here for your 
conveniance. Take them and install them onto your target hosts"

ask_title "Configuration of Linux/UNIX Agents"


ask_dir agentslibdir /usr/lib/check_mk_agent $HOMEBASEDIR/check_mk_agent "extensions for agents" \
  "This directory will not be created on the server. It will be hardcoded 
into the Linux and UNIX agents. The agent will look into this directory
for extensions"

ask_dir agentsconfdir /etc/check_mk $HOMEBASEDIR "configuration dir for agents" \
  "This directory will not be created on the server. It will be hardcoded
into the Linux and UNIX agents. The agent will look for its configuration
files here (currently only the logwatch extension needs a configuration file)"

ask_title "Integration with Nagios"

ask_dir htdocsdir /usr/share/check_mk/htdocs $HOMEBASEDIR/htdocs "Webpages" \
  "Directory for web pages and .php files. Current check_mk ships only 
one PHP file called logwatch.php. Make sure it's in the same directory
as the Nagios HTML files (the place where side.html and main.html reside)"

ask_dir -d nagiosuser nagios $(id -un) "Name of Nagios user" \
  "The working directory for check_mk contains several subdirectories
that need to be writable by the Nagios user (which is running check_mk 
in check mode). Please specify the user that should own those 
directories"

ask_dir -d wwwgroup nagios $(id -un) "Common group of Nagios+Apache" \
  "Check_mk creates files and directories while running as $nagiosuser. 
Some of those need to be writable by the user that is running the webserver.
Therefore a group is needed in which both Nagios and the webserver are
members (every valid Nagios installation uses such a group to allow
the web server access to Nagios' command pipe):"

ask_dir nagios_binary /usr/sbin/nagios $HOMEBASEDIR/nagios/bin/nagios "Nagios binary" \
  "The complete path to the Nagios executable. This is needed by the
option -R/--restart in order to do a configuration check."

ask_dir nagios_config_file /etc/nagios/nagios.cfg $HOMEBASEDIR/nagios/etc/nagios.cfg "Nagios main configuration file" \
  "Path to the main configuration file of Nagios. That file is always 
named 'nagios.cfg'. The default path when compiling Nagios yourself
is /usr/local/nagios/etc/nagios.cfg. The path to this file is needed
for the check_mk option -R/--restart"

ask_dir nagconfdir /etc/nagios/objects $HOMEBASEDIR/nagios/etc "Nagios object directory" \
  "Nagios' object definitions for hosts, services and contacts are
usually stored in various files with the extension .cfg. These files
are located in a directory that is configured in nagios.cfg with the
directive 'cfg_dir'. Please specify the path to that directory"

ask_dir nagios_startscript /etc/init.d/nagios /etc/init.d/nagios "Nagios startskript" \
  "The complete path to the Nagios startskript is used by the option
-R/--restart to restart Nagios."


ask_dir -d nagiosurl /nagios /nagios "Base URL of website" \
  "The configuration files generated by check_mk contain some
HTTP links to Nagios web pages. Please enter the correct base
URL of your Nagios pages. The trailing slash will be appended by check_mk"

ask_dir -d cgiurl /nagios/cgi-bin /nagios/cgi-bin "Base URL of cgi programs" \
  "The Nagios cgi programs have their own base URL. If you are
unsure then hower your mouse over a link to host or status 
data and look at your browser's status line. Leave out the
host name and also the 'status.cgi?...'"

ask_dir nagpipe /var/log/nagios/rw/nagios.cmd /var/log/nagios/rw/nagios.cmd "Nagios command pipe" \
  "Complete path to the Nagios command pipe. check_mk needs write access
to this pipe in order to operate"

ask_dir check_icmp_path /usr/lib/nagios/plugins/check_icmp $HOMEBASEDIR/libexec/check_icmp "Path to check_icmp" \
  "check_mk ships a Nagios configuration file with several host and
service templates. Some host templates need check_icmp as host check.
That check plugin is contained in the standard Nagios plugins.
Please specify the complete path to check_icmp"

ask_title "Integration with PNP4Nagios"

ask_dir rrddir $vardir/rrd $vardir/rrd "round robin databases" \
  "Base directory for round robin databases. If you use PNP4Nagios as
graphing tool check_mk can directly write into the exsting databases.
This saves CPU and disk IO"

ask_dir pnptemplates /usr/share/$NAME/pnp-templates $HOMEBASEDIR/pnp-templates "PNP4Nagios templates" \
  "check_mk ships templates for PNP4Nagios for most of its checks.
Those templates make the history graphs look nice. PNP4Nagios
expects such templates in the directory pnp/templates in your
document root for static web pages"

create_defaults ()
{
cat <<EOF
# This file has been created during setup
# of check_mk at $(date)
#
# Do not edit. If you want to use other
# directories please configure them in 
# main.mk
check_mk_version            = '$VERSION'
default_config_dir          = '$confdir'
check_mk_configdir          = '$confdir/conf.d'
checks_dir                  = '$checksdir'
check_manpages_dir          = '$checkmandir'
modules_dir                 = '$modulesdir'
var_dir                     = '$vardir'
autochecksdir               = '$vardir/autochecks'
precompiled_hostchecks_dir  = '$vardir/precompiled'
counters_directory          = '$vardir/counters'
tcp_cache_dir		    = '$vardir/cache'
nagios_objects_file         = '$nagconfdir/check_mk_objects.cfg'
rrd_path                    = '$rrddir'
nagios_command_pipe_path    = '$nagpipe'
nagios_conf_dir             = '$nagconfdir'
nagios_user                 = '$nagiosuser'
logwatch_notes_url          = '$nagiosurl/logwatch.php?host=%s&file=%s'
www_group                   = '$wwwgroup'
nagios_config_file          = '$nagios_config_file'
nagios_startscript          = '$nagios_startscript'
nagios_binary               = '$nagios_binary'
EOF
}

echo
echo "----------------------------------------------------------------------"
echo
echo "You have chosen the following directories: "
echo "$SUMMARY"
echo
echo
while true
do
    if [ -z "$DESTDIR" ] ; then
        read -p "Proceed with installation (y/n)? " JA
    else
	JA=yes
    fi
    case "$JA" in
        j|J|ja|Ja|JA|y|yes|Y|Yes|YES)
	   # Save paths for later installation
	   echo "$DIRINFO" > $SETUPCONF

	   mkdir -p $DESTDIR$modulesdir &&
	   create_defaults > $DESTDIR$modulesdir/defaults &&
	   mkdir -p $DESTDIR$checksdir &&
	   tar xzf $SRCDIR/checks.tar.gz -C $DESTDIR$checksdir &&
	   mkdir -p $DESTDIR$htdocsdir &&
	   tar xzf $SRCDIR/htdocs.tar.gz -C $DESTDIR$htdocsdir &&
	   mkdir -p $DESTDIR$pnptemplates &&
	   tar xzf $SRCDIR/pnp-templates.tar.gz -C $DESTDIR$pnptemplates &&
	   mkdir -p $DESTDIR$modulesdir &&
	   rm -f $DESTDIR$modulesdir/check_mk{,_admin} &&
	   tar xzf $SRCDIR/modules.tar.gz -C $DESTDIR$modulesdir &&
	   mkdir -p $DESTDIR$docdir &&
	   tar xzf $SRCDIR/doc.tar.gz -C $DESTDIR$docdir &&
	   mkdir -p $DESTDIR$checkmandir &&
	   tar xzf $SRCDIR/checkman.tar.gz -C $DESTDIR$checkmandir &&
	   if [ -e $SRCDIR/mibs.tar.gz ] ; then 
	       mkdir -p $DESTDIR$mibsdir &&
	       tar xzf $SRCDIR/mibs.tar.gz -C $DESTDIR$mibsdir
	   fi &&
	   mkdir -p $DESTDIR$agentsdir &&
	   tar xzf $SRCDIR/agents.tar.gz -C $DESTDIR$agentsdir &&
	   for agent in $DESTDIR/$agentsdir/* ; do 
	       sed -ri 's@^export MK_LIBDIR="(.*)"@export MK_LIBDIR="'"$agentslibdir"'"@' $agent 
	       sed -ri 's@^export MK_CONFDIR="(.*)"@export MK_CONFDIR="'"$agentsconfdir"'"@' $agent 
	   done &&
	   for php in $DESTDIR$htdocsdir/*.php ; do
	       sed -ri 's@\$vardir *= *"(.*)"@$vardir="'"$vardir"'"@' $php
           done &&
	   mkdir -p $DESTDIR$vardir/{autochecks,counters,precompiled,cache,logwatch} &&
	   if [ -z "$DESTDIR" ] && id "$nagiosuser" > /dev/null 2>&1 ; then
	     chown -R $nagiosuser $DESTDIR$vardir/{counters,cache,logwatch}
           fi &&
	   mkdir -p $DESTDIR$confdir/conf.d && 
	   tar xzf $SRCDIR/conf.tar.gz -C $DESTDIR$confdir &&
	   if [ -e $DESTDIR$confdir/check_mk.cfg -a ! -e $DESTDIR$confdir/main.mk ] ; then
	       mv -v $DESTDIR$confdir/check_mk.cfg $DESTDIR$confdir/main.mk
               echo "Renamed check_mk.cfg into main.mk." 
           fi &&
	   for f in $DESTDIR$vardir/autochecks/*.cfg $DESTDIR$confdir/conf.d/*.cfg ; do 
	       if [ -e "$f" ] ; then
		   mv -v $f ${f%.cfg}.mk 
               fi
           done &&
	   if [ ! -e $DESTDIR$confdir/main.mk ] ; then
	      tar xzf $SRCDIR/conf.tar.gz --to-stdout > $DESTDIR$confdir/main.mk
           fi &&
	   mkdir -p $DESTDIR$confdir/conf.d &&
	   echo 'All files in this directory that end with .mk will be read in after main.mk' > $DESTDIR$confdir/conf.d/README &&
	   if [ ! -d $DESTDIR$rrddir ] ; then
	       mkdir -p $DESTDIR$rrddir && 
	       if [ -z "$DESTDIR" ] && id "$nagiosuser" > /dev/null 2>&1 ; then
		   chown $nagiosuser $DESTDIR$rrddir
               fi
	   fi &&
	   mkdir -p $DESTDIR$bindir &&
	   rm -f $DESTDIR$bindir/check_mk &&
	   echo -e "#!/bin/sh\nexec python $modulesdir/check_mk.py "'"$@"' > $DESTDIR$bindir/check_mk
	   chmod 755 $DESTDIR$bindir/check_mk &&
	   sed -i "s#@BINDIR@#$bindir#g"              $DESTDIR$docdir/check_mk_templates.cfg &&
	   sed -i "s#@VARDIR@#$vardir#g"              $DESTDIR$docdir/check_mk_templates.cfg &&
	   sed -i "s#@CGIURL@#$cgiurl#g"              $DESTDIR$docdir/check_mk_templates.cfg &&
	   sed -i "s#@CHECK_ICMP@#$check_icmp_path#g" $DESTDIR$docdir/check_mk_templates.cfg &&
	   sed -i "s#@NAGIOSURL@#$nagiosurl#g"        $DESTDIR$docdir/check_mk_templates.cfg &&
	   if [ -d "$DESTDIR$nagconfdir" -o "$DESTDIR" ] ; then 
	       mkdir -p "$DESTDIR$nagconfdir"
	       ln -s $docdir/check_mk_templates.cfg $DESTDIR$nagconfdir 2>/dev/null
	   fi
	   echo "Installation completed successfully." ||
	   echo "ERROR!"
	   exit
        ;;
        n|N|no|No|Nein|nein)
        echo "Aborted."
        exit 1
        ;;
    esac
done
